package com.qzing.ime.abnl.abnlevent.application;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Sets;
import com.qzing.framework.core.utils.StringUtil;
import com.qzing.ieep.context.CurrentContextHelper;
import com.qzing.ieep.context.UserAuthInfo;
import com.qzing.ieep.data.common.RestResponse;
import com.qzing.ieep.util.CollectionUtils;
import com.qzing.ime.abnl.BaseAbnlResponse;
import com.qzing.ime.abnl.abnlevent.application.impl.StdAbnlEventAppService;
import com.qzing.ime.abnl.abnlevent.dao.AbnlEventDao;
import com.qzing.ime.abnl.abnlevent.dao.AbnlExtendDao;
import com.qzing.ime.abnl.abnlevent.dao.StdAbnlExtendDao;
import com.qzing.ime.abnl.abnlevent.entity.AbnlEvent;
import com.qzing.ime.abnl.abnlevent.entity.AbnlExtend;
import com.qzing.ime.abnl.abnlevent.entity.AbnlRecord;
import com.qzing.ime.abnl.abnlevent.model.constant.AbnlEventConstant;
import com.qzing.ime.abnl.abnlevent.model.enums.BusiStatusEnum;
import com.qzing.ime.abnl.abnlevent.model.enums.ProcessRecordEnum;
import com.qzing.ime.abnl.abnlevent.model.enums.ResponseResultEnum;
import com.qzing.ime.abnl.abnlevent.model.request.AbnlEventCloseRequest;
import com.qzing.ime.abnl.abnlevent.model.request.AbnlEventFinishRespondRequest;
import com.qzing.ime.abnl.abnlevent.model.request.AbnlEventStartHandleRequest;
import com.qzing.ime.abnl.abnlevent.model.request.EquipAbnlDataRequest;
import com.qzing.ime.abnl.abnluserrelation.model.enums.AbnlTypeEnum;
import com.qzing.ime.abnl.dashboard.dto.AbnlException;
import com.qzing.ime.abnl.dashboard.dto.IotEquipAbnlDto;
import com.qzing.ime.framework.toolkit.exception.ImeExceptionUtils;
import com.qzing.ime.framework.toolkit.utils.ImeI18NUtils;
import com.qzing.ime.framework.toolkit.utils.ImeLogger;
import com.qzing.toolkit.spring.api.QZingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.client.RestTemplate;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: yuzhao
 * @create: 2021-10-12
 **/
@QZingService(api = AbnlEventAppServiceImpl.class)
public class AbnlEventAppServiceImpl extends StdAbnlEventAppService implements AbnlEventAppService {
    private ImeLogger imeLogger = ImeLogger.getInstance(StdAbnlEventAppService.class, "al_abnl_event_relation");
    @Autowired
    private AbnlEventDao abnlEventDao;
    @Autowired
    private StdAbnlExtendDao stdAbnlExtendDao;

    @Autowired
    private AbnlExtendDao abnlExtendDao;

    @Autowired
    private RestTemplate restTemplate;

    @Value("${sdk.url}")
    private String sdkUrl;

    public BaseAbnlResponse closeAbnlEvent(AbnlEventCloseRequest request) {
        AbnlEvent abnlEvent = (AbnlEvent) this.abnlEventDao.getById(request.getGid());
        ImeExceptionUtils.isTrue(abnlEvent != null, ImeI18NUtils.getText("abnl_event_is_not_exist"));
//        ImeExceptionUtils.isTrue(abnlEvent.getBusiStatus().equals(BusiStatusEnum.FINISHHANDLE.name()), ImeI18NUtils.getText("abnl_event_can_not_close", new Object[]{abnlEvent.getCode()}));
        abnlEvent.setCloseDate(new Date());
        abnlEvent.setCloseRemark(request.getCloseRemark());
        abnlEvent.setBusiStatus(BusiStatusEnum.CLOSE.name());
        UserAuthInfo info = CurrentContextHelper.getUserAuthInfo();
        abnlEvent.setCloseUserCode(info.getUserCode());
        abnlEvent.setCloseUserName(info.getUserName());
        AbnlRecord abnlRecord = new AbnlRecord();
        abnlRecord.setProcessNode(ProcessRecordEnum.CLOSEABNL.name());
        abnlRecord.setExcuteDate(new Date());
        abnlRecord.setExcuteUserCode(info.getUserCode());
        abnlRecord.setExcuteUserName(info.getUserName());
        abnlRecord.setRemark(abnlEvent.getCloseRemark());
        abnlRecord.setPlantCode(abnlEvent.getPlantCode());
        abnlRecord.setPlantName(abnlEvent.getPlantName());
        abnlRecord.setCompanyCode(abnlEvent.getCompanyCode());
        abnlRecord.setCompanyName(abnlEvent.getCompanyName());
        abnlEvent.getAbnlRecordList().add(abnlRecord);
        AbnlEvent save = (AbnlEvent) this.abnlEventDao.save(abnlEvent);
        this.imeLogger.infoToDb(save.getGid(), save.getCode(), "异常事件关闭");
        return BaseAbnlResponse.success(save.getGid(), save.getCode(), "异常事件关闭");
    }

    @Override
    public BaseAbnlResponse finishRespond(AbnlEventFinishRespondRequest request) {
        AbnlEvent abnlEvent = abnlEventDao.getById(request.getGid());
        ImeExceptionUtils.isTrue(abnlEvent != null, ImeI18NUtils.getText(AbnlEventConstant.ABNL_EVENT_IS_NOT_EXIST));

        UserAuthInfo info = CurrentContextHelper.getUserAuthInfo();

        BeanUtil.copyProperties(request, abnlEvent);
        //状态：当响应结果为“受理异常”时，状态为“完成响应”，否则状态为“待响应”
        AbnlRecord abnlRecord = new AbnlRecord();
        if (request.getResponseResult().equals(ResponseResultEnum.ACCEPTANCE.name())) {
            abnlEvent.setBusiStatus(BusiStatusEnum.FINISHRESPONSE.name());
            abnlEvent.setFinishResponseDate(new Date());
            abnlRecord.setProcessNode(ProcessRecordEnum.FINISHRESPONSE.name());
            abnlEvent.setResponseUserCode(info.getUserCode());
            abnlEvent.setResponseUserName(info.getUserName());
        } else {
            //误判取消，状态改为关闭
            abnlEvent.setCloseDate(new Date());
            abnlEvent.setBusiStatus(BusiStatusEnum.CLOSE.name());
            abnlEvent.setCloseUserCode(info.getUserCode());
            abnlEvent.setCloseUserName(info.getUserName());
            abnlRecord.setProcessNode(ProcessRecordEnum.MISJUDGECANCEL.name());
        }
        abnlRecord.setExcuteDate(new Date());
        abnlRecord.setExcuteUserCode(info.getUserCode());
        abnlRecord.setExcuteUserName(info.getUserName());
        abnlRecord.setPlantCode(abnlEvent.getPlantCode());
        abnlRecord.setPlantName(abnlEvent.getPlantName());
        abnlRecord.setCompanyCode(abnlEvent.getCompanyCode());
        abnlRecord.setCompanyName(abnlEvent.getCompanyName());
        abnlRecord.setRemark(abnlEvent.getFinishResponseRemark());
        abnlEvent.getAbnlRecordList().add(abnlRecord);
        AbnlEvent save = abnlEventDao.save(abnlEvent);


        if (request.getResponseResult().equals(ResponseResultEnum.ACCEPTANCE.name())) {
            imeLogger.infoToDb(save.getGid(), save.getCode(), "异常事件响应完成");
        } else {
            imeLogger.infoToDb(save.getGid(), save.getCode(), "异常事件误判取消");
        }

        return BaseAbnlResponse.success(save.getGid(), save.getCode(), "异常事件响应完成");
    }

    @Override
    public BaseAbnlResponse starthandleAbnlEvent(AbnlEventStartHandleRequest request) {
        Long gid = request.getGid();
        AbnlEvent abnlEvent = abnlEventDao.getById(gid);
        Boolean sign = false;
        if (!abnlEvent.getBusiStatus().equals(BusiStatusEnum.FINISHRESPONSE.name())) {
            sign = true;
        }
        ImeExceptionUtils.isFalse(sign, AbnlEventConstant.ABNL_EVENT_CAN_NOT_HANDLE);
        //记录
        UserAuthInfo info = CurrentContextHelper.getUserAuthInfo();
        abnlEvent.setHandleFinishDate(request.getHandleFinishDate());
        abnlEvent.setHandleUserCode(info.getUserCode());
        abnlEvent.setHandleUserName(info.getUserName());
        abnlEvent.setHandleFinishRemark(request.getHandleFinishRemark());
        abnlEvent.setCloseDate(new Date());
        abnlEvent.setBusiStatus(BusiStatusEnum.CLOSE.name());
        abnlEvent.setHandleFinishDate(new Date());
        abnlEvent.setCloseUserCode(info.getUserCode());
        abnlEvent.setCloseUserName(info.getUserName());
        AbnlRecord abnlRecord = new AbnlRecord();
        abnlRecord.setExcuteDate(new Date());
        abnlRecord.setExcuteUserCode(info.getUserCode());
        abnlRecord.setExcuteUserName(info.getUserName());
        abnlRecord.setRemark(abnlEvent.getHandleFinishRemark());
        abnlRecord.setPlantCode(abnlEvent.getPlantCode());
        abnlRecord.setPlantName(abnlEvent.getPlantName());
        abnlRecord.setCompanyCode(abnlEvent.getCompanyCode());
        abnlRecord.setCompanyName(abnlEvent.getCompanyName());
        abnlRecord.setProcessNode(BusiStatusEnum.FINISHHANDLE.name());
        abnlEvent.getAbnlRecordList().add(abnlRecord);

        //关闭
        AbnlRecord abnlRecord2 = new AbnlRecord();
        abnlRecord2.setProcessNode(ProcessRecordEnum.CLOSEABNL.name());
        abnlRecord2.setExcuteDate(new Date());
        abnlRecord2.setExcuteUserCode(info.getUserCode());
        abnlRecord2.setExcuteUserName(info.getUserName());
        abnlRecord2.setRemark(abnlEvent.getCloseRemark());
        abnlRecord2.setPlantCode(abnlEvent.getPlantCode());
        abnlRecord2.setPlantName(abnlEvent.getPlantName());
        abnlRecord2.setCompanyCode(abnlEvent.getCompanyCode());
        abnlRecord2.setCompanyName(abnlEvent.getCompanyName());
        abnlEvent.getAbnlRecordList().add(abnlRecord2);


        AbnlEvent save = abnlEventDao.save(abnlEvent);

        imeLogger.infoToDb(save.getGid(), save.getCode(), "异常事件处置完成,关闭");

        return BaseAbnlResponse.success(save.getGid(), save.getCode(), "异常事件处置完成");
    }

    public RestResponse getEquipAbnlData(EquipAbnlDataRequest request) {

        List<Long> gidList = stdAbnlExtendDao.findOperationGidGidByCode(request.getWorkCenterCode());

        List<Map<String, String>> equipCodeByGidList = stdAbnlExtendDao.findEquipCodeByGidList(gidList);
        if (CollectionUtil.isEmpty(equipCodeByGidList)) {
            return RestResponse.error("无数据");
        }
        List<String> codeQueryList = new ArrayList<>();
        List<IotEquipAbnlDto> abnlQueryList = new ArrayList<>();
        HashMap<String, String> codeNameMap = new HashMap<>();
        for (Map<String, String> stringStringMap : equipCodeByGidList) {
            String equipCode = stringStringMap.get("equipCode");
            String equipName = stringStringMap.get("equipName");
            codeQueryList.add(equipCode);

            IotEquipAbnlDto iotEquipAbnlDto = new IotEquipAbnlDto();
            iotEquipAbnlDto.setEquipCode(equipCode);
            iotEquipAbnlDto.setEquipName(equipName);
            abnlQueryList.add(iotEquipAbnlDto);

            codeNameMap.put(equipCode, equipName);
        }
        //异常管理中的数据，设备管理
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("NOTIN_busiStatus", new String[]{BusiStatusEnum.CLOSE.name(), BusiStatusEnum.FINISHHANDLE.name()});
        paramMap.put("EQ_type", AbnlTypeEnum.EQUIPEXCP.name());
        List<AbnlEvent> abnlEventList = abnlEventDao.findAll(paramMap);

        List<IotEquipAbnlDto> abnlList = new ArrayList<>();
        for (AbnlEvent abnlEvent : abnlEventList) {
            Long abnlExtendGid = abnlEvent.getAbnlExtendGid();
            Optional<AbnlExtend> byId = abnlExtendDao.findById(abnlExtendGid);
            if (byId.isPresent()) {
                AbnlExtend abnlExtend = byId.get();
                String context = abnlExtend.getContext();
                IotEquipAbnlDto iotEquipAbnlDto = JSONObject.parseObject(context, IotEquipAbnlDto.class);
                if (codeQueryList.contains(iotEquipAbnlDto.getEquipCode())) {
                    iotEquipAbnlDto.setPhenomenonName(abnlEvent.getMainPhenomenonCode());
                    iotEquipAbnlDto.setPhenomenonCode(abnlEvent.getCode());
                    abnlList.add(iotEquipAbnlDto);
                }
            }
        }

        //异常管理中的数据，设备管理
//        Map<String, Object> paramMap2 = new HashMap<>();
//        paramMap2.put("NOTIN_busiStatus",new String[]{BusiStatusEnum.CLOSE.name(),BusiStatusEnum.FINISHHANDLE.name()});
//        paramMap2.put("EQ_type", AbnlTypeEnum.MATERIALEXCP.name());
//        List<AbnlEvent> materialList = abnlEventDao.findAll(paramMap2);
//
//        List<IotEquipAbnlDto> abnlMaterialList = new ArrayList<>();
//        for (IotEquipAbnlDto iotEquipAbnlDto : abnlQueryList) {
//            for (AbnlEvent abnlEvent : materialList) {
//
//                IotEquipAbnlDto iot = new IotEquipAbnlDto();
//                iot.setEquipCode(iotEquipAbnlDto.getEquipCode());
//                iot.setEquipName(iotEquipAbnlDto.getEquipName());
//                iot.setPhenomenonCode(abnlEvent.getCode());
//                iot.setPhenomenonName(abnlEvent.getMainPhenomenonCode());
//                abnlMaterialList.add(iot);
//            }
//        }
//
//
//        abnlList.addAll(abnlMaterialList);

//        List<IotEquipAbnlDto> iotList = new ArrayList<>();
//
//        Object response = restTemplate.postForObject(sdkUrl + "/alert/all", null, Object.class);
//        //Object response = restTemplate.postForObject( "http://120.202.38.15:8081/iot/edge/cxf/rongtong-edge-application/alert/all", null, Object.class);
//
//        JSONObject responseIot = JSONObject.parseObject(JSON.toJSONString(response));
//
//        if (ObjectUtils.equals(true, responseIot.get("success"))) {
//            List<IotResponseEquipAbnlTypeDto> iotResponse = JSON.parseArray(responseIot.get("data").toString(), IotResponseEquipAbnlTypeDto.class);
//
//            iotResponse.forEach(f->{
//                IotEquipAbnlDto d1 = new IotEquipAbnlDto();
//                BeanUtil.copyProperties(f,d1);
//                d1.setPhenomenonCode(f.getField());
//                iotList.add(d1);
//            });
//        }

//        List<IotEquipAbnlDto> removeList = new ArrayList<>();
//        for (IotEquipAbnlDto iotEquipAbnlDto : iotList) {
//            if (!codeQueryList.contains(iotEquipAbnlDto.getEquipCode())){
//                removeList.add(iotEquipAbnlDto);
//            }else {
//                iotEquipAbnlDto.setEquipName(codeNameMap.get(iotEquipAbnlDto.getEquipCode()));
//            }
//        }
//        iotList.removeAll(removeList);
//        abnlList.addAll(iotList);

        //abnlList去重
        // 根据设备编码，异常现象编码两个属性去重
//        abnlList = abnlList.stream().collect(
//                Collectors. collectingAndThen(
//                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o -> o.getEquipCode() + ";" + o.getPhenomenonCode()))), ArrayList::new)
//        );

        //设备编码,设备名称相同的合并phenomenon
//        List<IotEquipAbnlDto> allList=new ArrayList<>();
//        abnlList.parallelStream().collect(Collectors.groupingBy(o -> (o.getEquipCode()+o.getEquipName()),Collectors.toList())).forEach((id,transfer)->{
//            transfer.stream().reduce((a,b)->new IotEquipAbnlDto(a.getEquipCode(),a.getEquipName(),a.getPhenomenonCode()+","+b.getPhenomenonCode(),a.getPhenomenonName()+","+b.getPhenomenonName())).ifPresent(allList::add);
//        });

        return RestResponse.success("获取设备异常数据成功", abnlList);
    }

    public RestResponse getUnplannedStopData(EquipAbnlDataRequest request) {
        List<Long> gidList = stdAbnlExtendDao.findOperationGidGidByCode(request.getWorkCenterCode());

        List<Map<String, String>> equipCodeByGidList = stdAbnlExtendDao.findEquipCodeByGidList(gidList);
        if (CollectionUtil.isEmpty(equipCodeByGidList)) {
            return RestResponse.error("无数据");
        }
        List<String> codeQueryList = new ArrayList<>();
        List<IotEquipAbnlDto> abnlQueryList = new ArrayList<>();
        for (Map<String, String> stringStringMap : equipCodeByGidList) {
            String equipCode = stringStringMap.get("equipCode");
            String equipName = stringStringMap.get("equipName");
            codeQueryList.add(equipCode);

            IotEquipAbnlDto iotEquipAbnlDto = new IotEquipAbnlDto();
            iotEquipAbnlDto.setEquipCode(equipCode);
            iotEquipAbnlDto.setEquipName(equipName);
            abnlQueryList.add(iotEquipAbnlDto);
        }

        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<String> pastDayList = pastDay(sdf.format(date));

        Map<String, Long> map = new HashMap<>();
        pastDayList.forEach(c -> map.put(c, 0l));
        for (String s : pastDayList) {
            String afterDate = getAfterDate(s);

            String hsql = "FROM AbnlEvent  WHERE type='EQUIPEXCP' and createTime>'" + s + "' and createTime<'" + afterDate + "'";
            //以工序作业完成时间为基准，查询当天
            List<AbnlEvent> abnlEventList = abnlEventDao.executeQuery(hsql);


            if (CollectionUtils.isEmpty(abnlEventList)) continue;
            long sum = 0l;
            for (AbnlEvent abnlEvent : abnlEventList) {
                Long abnlExtendGid = abnlEvent.getAbnlExtendGid();
                Optional<AbnlExtend> byId = abnlExtendDao.findById(abnlExtendGid);
                if (!byId.isPresent()) continue;
                AbnlExtend abnlExtend = byId.get();
                String context = abnlExtend.getContext();
                IotEquipAbnlDto iotEquipAbnlDto = JSONObject.parseObject(context, IotEquipAbnlDto.class);

                if (!codeQueryList.contains(iotEquipAbnlDto.getEquipCode())) continue;

                Date createTime = abnlEvent.getCreateTime().getTime();
                Date closeDate = abnlEvent.getCloseDate();
                Date handleFinishDate = abnlEvent.getHandleFinishDate();
                Date nowDate = new Date();
                if (abnlEvent.getBusiStatus().equals(BusiStatusEnum.CLOSE.name())) {
                    sum = sum + (closeDate.getTime() - createTime.getTime());
                } else if (abnlEvent.getBusiStatus().equals(BusiStatusEnum.FINISHHANDLE.name())) {
                    sum = sum + (handleFinishDate.getTime() - createTime.getTime());
                } else {
                    sum = sum + (nowDate.getTime() - createTime.getTime());
                }
            }
            map.put(s, sum);
        }

        return RestResponse.success("获取设备异常数据成功", map);
    }

    /**
     * 获取过去第几天的日期
     *
     * @param past
     * @return
     */
    public static String getPastDate(int past, Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - past);
        Date today = calendar.getTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String result = sdf.format(today);
        return result;
    }

    /**
     * 获取过去7天内的日期数组
     *
     * @param time
     * @return 日期数组
     */
    public static ArrayList<String> pastDay(String time) {
        ArrayList<String> pastDaysList = new ArrayList<>();
        try {
            //我这里传来的时间是个string类型的，所以要先转为date类型的。
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date date = sdf.parse(time);
            for (int i = 7; i >= 1; i--) {
                pastDaysList.add(getPastDate(i, date));
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return pastDaysList;
    }


    public static String getAfterDate(String time) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date date = null;
        try {
            date = sdf.parse(time);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) + 1);
        Date today = calendar.getTime();
        String result = sdf.format(today);
        return result;
    }

    public RestResponse getAbnlException(String workCenterCode) {
        List<AbnlException> listException = new ArrayList<>();
        //异常管理中的数据
        Map<String, Object> paramMap = new HashMap<>();
        List<String> codeList = stdAbnlExtendDao.findWorkCenterCodesByCode(workCenterCode);
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String format = sdf.format(date);

        paramMap.put("NOTIN_busiStatus", new String[]{BusiStatusEnum.CLOSE.name(), BusiStatusEnum.FINISHHANDLE.name()});

        paramMap.put("GE_createTime",format);
        String[] dataCode = codeList.toArray(new String[codeList.size()]);


        paramMap.put("IN_workCenterCode", dataCode);
        LinkedHashSet<IotEquipAbnlDto> threeAbnlException = this.findThreeAbnlException(workCenterCode, paramMap);

        return RestResponse.success(threeAbnlException);
    }


    /**
     * 查询三种异常
     */


    public LinkedHashSet<IotEquipAbnlDto> findThreeAbnlException(String workCenterCode, Map<String, Object> paramMap) {
        LinkedHashSet<IotEquipAbnlDto> set = new LinkedHashSet<>();

        String[] types = {AbnlTypeEnum.MATERIALEXCP.name(), AbnlTypeEnum.ROUTEEXCP.name(), AbnlTypeEnum.PLANEXCP.name()};
        List<AbnlEvent> abnlEventList = abnlEventDao.findAll(paramMap);
        if (CollectionUtil.isNotEmpty(abnlEventList)) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

            for (AbnlEvent abnlEvent : abnlEventList) {
                Long abnlExtendGid = abnlEvent.getAbnlExtendGid();
                Optional<AbnlExtend> byId = abnlExtendDao.findById(abnlExtendGid);
                if (byId.isPresent()) {
                    AbnlExtend abnlExtend = byId.get();
                    String context = abnlExtend.getContext();
                    IotEquipAbnlDto iotEquipAbnlDto = JSONObject.parseObject(context, IotEquipAbnlDto.class);
                    //设备异常
                    if (abnlEvent.getType().equals(AbnlTypeEnum.EQUIPEXCP.toString())) {
                        iotEquipAbnlDto.setPhenomenonName(abnlEvent.getMainPhenomenonCode());
                        iotEquipAbnlDto.setPhenomenonCode(abnlEvent.getCode());
                        Date today = abnlEvent.getCreateTime().getTime();
                        String resultTime = sdf.format(today);
                        resultTime=resultTime.substring(5);
                        iotEquipAbnlDto.setTime(resultTime);
                        iotEquipAbnlDto.setTypeException("equip");
                        set.add(iotEquipAbnlDto);

                    }

                }
            }
            abnlEventList.stream().forEach(t -> {
                if (t.getType().equals(AbnlTypeEnum.QUALITYEXCP.toString())) {
                    IotEquipAbnlDto iotEquipAbnlDto = new IotEquipAbnlDto();
                    iotEquipAbnlDto.setPhenomenonName(t.getMainPhenomenonCode());
                    iotEquipAbnlDto.setPhenomenonCode(t.getCode());
                    Date today = t.getCreateTime().getTime();
                    String resultTime = sdf.format(today);
                    resultTime=resultTime.substring(5);
                    iotEquipAbnlDto.setTime(resultTime);
                    iotEquipAbnlDto.setTypeException("quality");
                    set.add(iotEquipAbnlDto);
                }
            });

            abnlEventList.stream().forEach(m -> {
                if (Arrays.asList(types).contains(m.getType())) {
                    IotEquipAbnlDto iotEquipAbnlDto = new IotEquipAbnlDto();
                    iotEquipAbnlDto.setPhenomenonName(m.getMainPhenomenonCode());
                    iotEquipAbnlDto.setPhenomenonCode(m.getCode());
                    Date today = m.getCreateTime().getTime();
                    String resultTime = sdf.format(today);
                    resultTime=resultTime.substring(5);
                    iotEquipAbnlDto.setTime(resultTime);
                    iotEquipAbnlDto.setTypeException("prodexec");
                    set.add(iotEquipAbnlDto);
                }
            });

        }
        return set;
    }

    public Map<String, List<AbnlEvent>> getAbnlList(Map<String,String> abnlParam) {
        Map<String, List<AbnlEvent>> results = new HashMap<>();
        Map<String, Object> params = new HashMap<>();
        params.put("IN_sfcCode" , abnlParam.keySet());
        List<AbnlEvent> abnlEventList = abnlEventDao.findAllWithoutAssociation(params);

        if(!CollectionUtils.isEmpty(abnlEventList)){
            Map<String, List<AbnlEvent>> sfcCodeMap = abnlEventList.stream().collect(Collectors.groupingBy(abnlEvent -> abnlEvent.getSfcCode()));
            sfcCodeMap.forEach((k,v) -> {
                results.put(k + "_" + abnlParam.get(k) , v);
            });

            Set<String> sfcCodes = abnlEventList.stream().map(abnlEvent -> abnlEvent.getSfcCode()).collect(Collectors.toSet());
            Sets.SetView<String> difference = Sets.difference(abnlParam.keySet(), sfcCodes);
            if(difference.size() >= 0){
                HashMap<String, String> workOrderCodeParam = new HashMap<>();
                difference.forEach(sfcCode -> {
                    String workOrderCode = abnlParam.get(sfcCode);
                    if(StringUtil.isNotEmpty(workOrderCode)){
                        workOrderCodeParam.put(workOrderCode , sfcCode);
                    }
                });

                params.clear();
                params.put("IN_workOrderCode" , workOrderCodeParam.keySet());
                abnlEventList = abnlEventDao.findAllWithoutAssociation(params);
                if(!CollectionUtils.isEmpty(abnlEventList)){
                    Map<String, List<AbnlEvent>> workOrderCodeMap = abnlEventList.stream().collect(Collectors.groupingBy(abnlEvent -> abnlEvent.getWorkOrderCode()));
                    workOrderCodeMap.forEach((k,v) -> {
                        results.put(workOrderCodeParam.get(k) + "_" + k, v);
                    });
                }
            }
        }else{
            params.clear();
            params.put("IN_workOrderCode" , abnlParam.values());
            abnlEventList = abnlEventDao.findAllWithoutAssociation(params);
            if(!CollectionUtils.isEmpty(abnlEventList)){
                Map<String, List<AbnlEvent>> workOrderCodeMap = abnlEventList.stream().collect(Collectors.groupingBy(abnlEvent -> abnlEvent.getWorkOrderCode()));
                abnlParam.forEach((k,v) -> {
                    if(!CollectionUtils.isEmpty(workOrderCodeMap.get(v))){
                        results.put(k + "_" + v, workOrderCodeMap.get(v));
                    }
                });
            }
        }
        return results;
    }
}
