package com.qzing.ime.abnl.earlywarningnotice.application;

import cn.hutool.core.util.ObjectUtil;
import com.qzing.ieep.data.common.RestResponse;
import com.qzing.ime.abnl.earlywarningnotice.dao.EarlyWarningNoticeDao;
import com.qzing.ime.abnl.earlywarningnotice.entity.EarlyWarningNotice;
import com.qzing.ime.abnl.earlywarningnotice.entity.EarlyWarningNoticeDetail;
import com.qzing.ime.abnl.earlywarningnotice.model.request.EarlyWarningNoticeCreateRequest;
import com.qzing.ime.abnl.earlywarningnotice.model.request.EarlyWarningNoticeUpdateRequest;
import com.qzing.ime.abnl.earlywarningnotice.model.response.EarlyWarningNoticeResponse;
import com.qzing.ime.framework.toolkit.exception.ImeExceptionUtils;
import com.qzing.scheduler.api.CloudJobClient;
import com.qzing.scheduler.dto.CloudJobDto;
import com.qzing.scheduler.dto.CloudJobState;
import com.qzing.scheduler.entity.CloudJob;
import com.qzing.toolkit.spring.api.QZingService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author DWJ
 */
@QZingService(api = EarlyWarningNoticeService.class)
@Slf4j
public class EarlyWarningNoticeServiceImpl implements EarlyWarningNoticeService {

    @Autowired
    private EarlyWarningNoticeDao earlyWarningNoticeDao;

    @Autowired
    private CloudJobClient cloudJobClient;

    @Override
    public RestResponse create(EarlyWarningNoticeCreateRequest request) {
        EarlyWarningNotice oEarlyWarningNotice = earlyWarningNoticeDao.findFirstByCode(request.getCode());
        if (!ObjectUtils.isEmpty(oEarlyWarningNotice)) {
            throw new IllegalArgumentException("已存在当前编码的预警管理配置");
        }
        EarlyWarningNotice earlyWarningNotice = new EarlyWarningNotice();
        BeanUtils.copyProperties(request, earlyWarningNotice);
        earlyWarningNotice.setStatus("2");
        earlyWarningNoticeDao.save(earlyWarningNotice);
        return RestResponse.success();
    }

    @Override
    public RestResponse modify(EarlyWarningNoticeUpdateRequest request) {
        EarlyWarningNotice earlyWarningNotice = earlyWarningNoticeDao.findByGid(request.getGid());
        BeanUtils.copyProperties(request, earlyWarningNotice);
        earlyWarningNoticeDao.save(earlyWarningNotice);
        return RestResponse.success();
    }

    @Override
    public RestResponse delete(List<Long> gids) {
        List<CloudJobDto> deleteJobIds = new ArrayList<>();
        gids.forEach(f -> {
            //删除实体
            EarlyWarningNotice earlyWarningNotice = earlyWarningNoticeDao.findByGid(f);
            if (ObjectUtil.isNotEmpty(earlyWarningNotice.getJobId())) {
                CloudJobDto cloudJobId = new CloudJobDto();
                cloudJobId.setJobId(earlyWarningNotice.getJobId());
                RestResponse byId = this.cloudJobClient.getById(cloudJobId);
                if (ObjectUtil.isNotEmpty(byId.getData())) {
                    deleteJobIds.add(cloudJobId);
                }
            }
            earlyWarningNoticeDao.delete(earlyWarningNotice);
        });
        //删除定时任务
        if (!CollectionUtils.isEmpty(deleteJobIds)) {
            RestResponse restDeleteResponse = this.cloudJobClient.deleteById(deleteJobIds);
            if (!restDeleteResponse.getCode().equals("0000")) {
                ImeExceptionUtils.error("平台删除定时任务失败！");
            }
        }
        return RestResponse.success();
    }

    @Override
    public RestResponse enable(EarlyWarningNoticeUpdateRequest request) {
        EarlyWarningNotice earlyWarningNotice = earlyWarningNoticeDao.findByGid(request.getGid());
        //及时类型的直接存值，不做定时任务
        if ("TIMELY".equals(earlyWarningNotice.getDetailRateType())) {
            earlyWarningNotice.setStatus(request.getStatus());
            earlyWarningNoticeDao.save(earlyWarningNotice);
            return RestResponse.success();
        }
        if ("1".equals(request.getStatus())) {
            //启用
            if (ObjectUtil.isNotEmpty(earlyWarningNotice.getJobId())) {
                //不为空则恢复相应的定时任务
                CloudJobState cloudJobState = new CloudJobState();
                cloudJobState.setJobId(earlyWarningNotice.getJobId());
                RestResponse response = cloudJobClient.resumeById(cloudJobState);
                if (!response.getCode().equals("0000")) {
                    ImeExceptionUtils.error("定时任务修改失败！");
                }
            } else {
                //为空则创建定时任务
                CloudJob cloudJob = new CloudJob();
                //设置启用和名称
                cloudJob.setStatus(true);
                cloudJob.setName(earlyWarningNotice.getName() + "定时任务");
                String detailRateType = earlyWarningNotice.getDetailRateType();
                //执行频率类型（“FIXED：固定间隔，DETAIL：明细设置，4：CRON：表达式设置”）
                cloudJob.setFrequencyType(detailRateType);
                //固定间隔 - 定时执行类型：SECOND：秒，MINUTE：分钟，HOUR：时，DAILY：天
                if ("FIXED".equals(detailRateType)) {
                    cloudJob.setDetailRateType("DAILY");
                    cloudJob.setFixedRateType(earlyWarningNotice.getFixedRateType());
                    cloudJob.setFixedRateValue(earlyWarningNotice.getFixedRateValue());
                    //cloudJob.setHours(earlyWarningNotice.getHours());
                    //cloudJob.setMinutes(earlyWarningNotice.getMinutes());
                } else if ("DETAIL".equals(detailRateType)) {
                    //明细类型的记录执行小时数和执行分钟数（例如每天8点执行一次的）
                    cloudJob.setDetailRateType("DAILY");
                    cloudJob.setDetailHour(earlyWarningNotice.getDetailHour());
                    cloudJob.setDetailMinute(earlyWarningNotice.getDetailMinute());
                }
                //设置开始时间
                if (!ObjectUtils.isEmpty(earlyWarningNotice.getStartTime())) {
                    cloudJob.setStartTime(earlyWarningNotice.getStartTime());
                    cloudJob.setStartTimeFlag(false);
                } else {
                    //没有开始时间设置为立即执行
                    cloudJob.setStartTimeFlag(true);
                }
                //设置结束时间
                if (!ObjectUtils.isEmpty(earlyWarningNotice.getEndTime())) {
                    cloudJob.setEndTime(earlyWarningNotice.getEndTime());
                    //不持续执行
                    cloudJob.setEndTimeFlag(false);
                } else {
                    //持续执行
                    cloudJob.setEndTimeFlag(true);
                }
                //设置服务名称
                String[] codes = earlyWarningNotice.getCode().split("#");
                cloudJob.setServiceName(codes[0]);
                //设置保存记录
                cloudJob.setWriteBizHistory("1");
                cloudJob.setWriteHistory("1");

                //设置调度调用接口和参数
                cloudJob.setUri(codes[1]);
                cloudJob.setRetry(0);
                cloudJob.setWriteBizHistory("0");
                cloudJob.setWriteHistory("0");
                RestResponse restResponse = cloudJobClient.saveOrUpdate(cloudJob);
                String jobId = (String) restResponse.getData();
                //更新定时调度id到质检单模板
                earlyWarningNotice.setJobId(Long.valueOf(jobId));
            }
            earlyWarningNotice.setStatus("1");
        } else if ("2".equals(request.getStatus())) {
            //停用
            CloudJobState cloudJobState = new CloudJobState();
            cloudJobState.setJobId(earlyWarningNotice.getJobId());
            RestResponse response = cloudJobClient.pausedById(cloudJobState);
            if (!response.getCode().equals("0000")) {
                ImeExceptionUtils.error("定时任务停用失败！");
            }
            earlyWarningNotice.setStatus("2");
        }
        earlyWarningNoticeDao.save(earlyWarningNotice);
        return RestResponse.success();
    }

    @Override
    public RestResponse findByCode(String code) {
        EarlyWarningNotice earlyWarningNotice = earlyWarningNoticeDao.findFirstByCode(code);
        if (ObjectUtils.isEmpty(earlyWarningNotice)) {
            return RestResponse.error("未查询到相关配置");
        } else {
            EarlyWarningNoticeResponse earlyWarningNoticeResponse = new EarlyWarningNoticeResponse();
            BeanUtils.copyProperties(earlyWarningNotice, earlyWarningNoticeResponse);
            return RestResponse.success(earlyWarningNoticeResponse);
        }
    }

}
