package com.qzing.ime.abnl;


import com.qzing.ieep.data.common.RestResponse;
import com.qzing.ime.framework.toolkit.exception.ImeException;
import com.qzing.ime.framework.toolkit.exception.ImeImportException;
import com.qzing.ime.framework.toolkit.utils.ImeI18NUtils;
import com.qzing.toolkit.spring.controller.request.VlidateFailedException;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import javax.validation.ConstraintViolation;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

@ControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class AbnlGlobalErrorController {

    static final  String STATUS_CODE = "10000";

    @ExceptionHandler({Exception.class})
    @ResponseBody
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public RestResponse handleException(Exception e) {
        RestResponse restResponse = RestResponse.error(HttpStatus.INTERNAL_SERVER_ERROR.toString(), e.getMessage(), Collections.EMPTY_MAP);
        e.printStackTrace();
        return restResponse;
    }

    @ExceptionHandler({ImeImportException.class})
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    public RestResponse handleException(ImeImportException e) {
        return RestResponse.error(String.valueOf(HttpStatus.INTERNAL_SERVER_ERROR.value()),e.getMessage());
    }

    @ExceptionHandler({ImeException.class})
    @ResponseBody
    @ResponseStatus(HttpStatus.OK)
    public RestResponse handleException(ImeException e) {
        Map<String, Object> extra = new HashMap<>();
        extra.put("imeException", true);
        extra.put("imeExceptionCode", STATUS_CODE);
        extra.put("imeExceptionMessage", e.getMessage());
        return RestResponse.error(STATUS_CODE,e.getMessage(),extra);
    }

    /**
     * 处理实体字段校验不通过异常
     * @param ex
     * @return
     */
    @ExceptionHandler(VlidateFailedException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    @ResponseBody
    public RestResponse validationError(VlidateFailedException ex) {
        StringBuilder stringBuilder = new StringBuilder();
        for (ConstraintViolation constraintViolation : ex.getConstraintViolationSet()){
            stringBuilder.append(ImeI18NUtils.getText(constraintViolation.getMessageTemplate())+";");
        }
        RestResponse result = RestResponse.error(HttpStatus.INTERNAL_SERVER_ERROR.toString(), stringBuilder.toString(), Collections.EMPTY_MAP);
        ex.printStackTrace();
        return result;
    }
}
